/*
 * Interface with PROATM NDIS5 driver for XP and previous Windows OS
 *
 */

#define _WINSOCKAPI_ /* Prevent inclusion of winsock.h in windows.h */

#include <windows.h>
#include <winsock2.h>
#include <ws2atm.h>
//#include <ansi_c.h>  // for LabWindows environment
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <Iphlpapi.h>
#include "public.h"
#include "proatmapi.h"
#include "ndisatmif.h"

#define MAX_ADAPTER_LIST   32
#define OS_UNKNOWN  -1
#define OS_WIN31    1
#define OS_WIN9X    2
#define OS_WINNT4   3
#define OS_WIN2K    4
#define OS_WINXP    5
#define OS_WINVISTA 6
#define OS_WINSEVEN 7
#define OS_WINEIGHT 8
#define OS_WINTEN	10


typedef	DWORD (CALLBACK* GET_PER_ADAPTER_INFO)( ULONG , VOID *, ULONG * );

#define LONG_MAX_NAME_LENGTH 256
/* definitions of network interface structure (one per interface)	*/
typedef struct _NET_INTERFACE                   /* info about one net interface	*/
{				
	struct _NET_INTERFACE    *pNext;
	HANDLE			hFile;         
	int				AdapterType;
	ULONG			ProtocolBindingContext;         
	int				NetDeviceCount;
	OVERLAPPED		ReadOvlp;
	OVERLAPPED		WriteOvlp;
	BYTE			*CurrentReceivePtr;
	UCHAR			State;		            /* interface states: NIS_above	*/
	BOOL			bCreatedDevice;
	char			IdentString[LONG_MAX_NAME_LENGTH];
	char			DriverDesc[LONG_MAX_NAME_LENGTH];
	BYTE			LowerLayer[32];
	BYTE			MACName[32];
	MACADDRESS		MACAddress;
	DWORD			IPAddress;
	DWORD			IPMask;

} NET_INTERFACE, *PNET_INTERFACE, **PPNET_INTERFACE;

int GetOSType(void)
{
	OSVERSIONINFO Info;

	int Type;

	memset(&Info, 0, sizeof(Info));
	Info.dwOSVersionInfoSize = sizeof(Info);
	GetVersionEx(&Info);

	if ( Info.dwPlatformId == VER_PLATFORM_WIN32s ) // Windows 3.1
		Type = OS_WIN31;

	else if ( Info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
		Type = OS_WIN9X;		// Indicate to Driver code that OS is 95

	else if (Info.dwMajorVersion == 4)
		Type = OS_WINNT4;

	else if (Info.dwMajorVersion == 5)  
	{ 
		//Windows NT 5.0 (2000 - XP)
		if (Info.dwMinorVersion == 0)
			Type = OS_WIN2K;
		else
			Type = OS_WINXP;
	}  

	else if (Info.dwMajorVersion == 6)  
	{
		//Windows NT 6.0 ( VISTA - EIGHT)
		if (Info.dwMinorVersion == 0)
			Type = OS_WINVISTA;

		else if (Info.dwMinorVersion == 1)
			Type = OS_WINSEVEN;

		else if (Info.dwMinorVersion == 2)
			Type = OS_WINEIGHT;
		else 
			Type = OS_UNKNOWN;

	}
	else if (Info.dwMajorVersion == 10)  
	{
		Type = OS_WINTEN;
	}
	
	else
		Type = OS_UNKNOWN;

	return Type;

}

// Initializes WinSock2
static BOOL InitWS2(void)
{
	WORD ver_req;
	WSADATA wsa_data;

	ver_req = MAKEWORD(2, 2);

	if (WSAStartup(ver_req, &wsa_data) != 0)
	{
		printf("Error : Starting up WinSock2.0 DLL failed. Code:%d\n", WSAGetLastError());
		return FALSE;
	}

	if (LOBYTE(wsa_data.wVersion) != 2 || HIBYTE(wsa_data.wVersion) != 2)
	{
		printf("Error : Invalid winsock version :%d.%d\n", LOBYTE(wsa_data.wVersion), HIBYTE(wsa_data.wVersion));
		WSACleanup();
		return FALSE;
	}

	return TRUE;

}

BOOL NDISGetATMAdapters(ATM_ADAPTER pAdapters[], DWORD *pAdapterCount)
{
	DWORD i;
	SOCKET sock;
	DWORD bytes;
	DWORD AdapterCount, DeviceCount;

	AdapterCount = *pAdapterCount;

	if (GetOSType() > OS_WINXP) 
	{
		return FALSE;
	}

	if (!InitWS2())
	{
		return FALSE;
	}
	if ((sock = WSASocket(AF_ATM, SOCK_RAW, ATMPROTO_AAL5, NULL, 0, 0))	== INVALID_SOCKET) 
	{
		printf("WSASocket failed: %d\n", WSAGetLastError());
		return FALSE;
	}

	/* Check available ATM devices */
	if(WSAIoctl(sock,
		SIO_GET_NUMBER_OF_ATM_DEVICES,
		NULL,
		0,
		&DeviceCount,
		sizeof(DWORD),
		&bytes,
		NULL,
		NULL) != 0)
	{
		printf("WSAIoctl(SIO_GET_NUMBER_OF_ATM_DEVICES..) failed: %d\n", WSAGetLastError());
		return FALSE;
	}
	
	for (i= AdapterCount; i < AdapterCount+DeviceCount; i++)
	{
		memset(&pAdapters[i], 0, sizeof(ATM_ADAPTER));
		pAdapters[i].AdapterType = NDIS_ADAPTER;
		pAdapters[i].DeviceNumber = i-AdapterCount;
	}
	closesocket(sock);

	WSACleanup();
	(*pAdapterCount) = (*pAdapterCount) + DeviceCount; 

	return TRUE;

}

BOOL NDISGetATMInformation ( PATM_ADAPTER pAdapter)
{
	return TRUE;
}

HANDLE NDISOpenATMSocket( PATM_ADAPTER pAdapter)
{
	SOCKET sock;

	if (!InitWS2())
		return INVALID_HANDLE_VALUE;

	sock = WSASocket(AF_ATM, SOCK_RAW, ATMPROTO_AAL5, NULL, NO_GROUP, WSA_FLAG_OVERLAPPED);	

	return (HANDLE)sock;
}

int	NDISCloseATMSocket( HANDLE hSocket)
{
	closesocket((SOCKET)hSocket);
	return 0;
}


static DWORD NDISReceiveThread(LPVOID param)
{
	DWORD  bytesRecv;
	int i,iError;
	DWORD dwEvent;
	int RecvFlags = 0 ;

	WSAOVERLAPPED wsaRecvOverlapped[MAX_RECV_PDUS +1];
	HANDLE	      hRecvEventList[MAX_RECV_PDUS +1];
	PPDU          pRecvPDU[MAX_RECV_PDUS];  //
	WSABUF		  wsaBuf[MAX_RECV_PDUS];


	PATM_PVC pVC	= (PATM_PVC)param;
	SOCKET hSocket	= (SOCKET)pVC->hSocket;
	UINT MaxPacket	= pVC->ATMSettings.PacketSize;
	PLIST_HEAD pReceiveQueue   = &pVC->ReceiveQueue;
	int	CurrentIndex=0;

	iError = 0;	

	for (i = 0; i<MAX_RECV_PDUS+1; i++)
	{
		memset(&wsaRecvOverlapped[i],0, sizeof(OVERLAPPED));
		hRecvEventList[i] = NULL;
		if (i < MAX_RECV_PDUS)
		{
			pRecvPDU[i]= NULL;
			wsaBuf[i].buf = NULL;
			wsaBuf[i].len = 0;
		}
	}

	for (i=0; i < MAX_RECV_PDUS; i++)
	{
		wsaRecvOverlapped[i].hEvent= WSACreateEvent();
		if (wsaRecvOverlapped[i].hEvent == NULL)
		{ 
			iError = WSAGetLastError();
			printf("WSACreateEvent failed: %d\n", iError);
			break;
		}
		if ((pRecvPDU[i] = (PPDU)calloc(1, sizeof(PDU) + MaxPacket )) == NULL)
		{
			iError = ERROR_NOT_ENOUGH_MEMORY;
			printf("Memory allocation failed\n");
			break;
		}
		pRecvPDU[i]->pBuffer = ((char *)(pRecvPDU[i]) +sizeof(PDU));
		pRecvPDU[i]->uLength = MaxPacket;
		RecvFlags = 0 ;
		wsaBuf[i].len = pRecvPDU[i]->uLength;
		wsaBuf[i].buf = pRecvPDU[i]->pBuffer;
		
		// Initialize the read of all VCs 
		if ( WSARecv(hSocket, &wsaBuf[i], 1, NULL, &RecvFlags, &wsaRecvOverlapped[i], NULL) != 0)
		{
			iError = WSAGetLastError();
			if (iError != WSA_IO_PENDING)
			{
				printf("WSARecv failed: %d\n", iError);
				break;
			}
			hRecvEventList[i]=wsaRecvOverlapped[i].hEvent;
		}
	}

	if (iError == 0 || iError == ERROR_IO_PENDING)
	{
		CurrentIndex = 0;
		hRecvEventList[MAX_RECV_PDUS]=pVC->hReceiveCloseEvent;
		while(TRUE)
		{
			// Wait for end of reads or close events
			dwEvent = WSAWaitForMultipleEvents( MAX_RECV_PDUS+1 - CurrentIndex , 
									&hRecvEventList[CurrentIndex] , FALSE, WSA_INFINITE, FALSE);

			if (dwEvent <  (WAIT_OBJECT_0 + MAX_RECV_PDUS - CurrentIndex))
			{ 
				int bReceived = FALSE;
				CurrentIndex += dwEvent;
				iError = 0;
				
				// There is at least one event signaled about one of the provided buffers.
				// Check all buffers. The index of the first signaled buffer is dwEvent.
				// Check events without waiting. If the operation is still pending, GetOverlappedResult () 
				// will return FALSE and GetLastError () will return WSA_IO_INCOMPLETE.  
				
				while (WSAGetOverlappedResult( hSocket, &wsaRecvOverlapped[CurrentIndex], 
															&bytesRecv,FALSE, &RecvFlags ) == TRUE )
				{
					PPDU pPDU=pRecvPDU[CurrentIndex];
					pRecvPDU[CurrentIndex] = NULL;
					pPDU->uLength = bytesRecv;
					MUTEX_LOCK(pVC->hReceiveMutex);
					LIST_ADD_TAIL( &pPDU->Link, pReceiveQueue);
					MUTEX_UNLOCK(pVC->hReceiveMutex);

					if ((pRecvPDU[CurrentIndex] = (PPDU)calloc( 1, sizeof(PDU) + MaxPacket ) ) == NULL)
					{
						printf("Memory allocation failed\n");
						iError = ERROR_NOT_ENOUGH_MEMORY;
						break;
					}
					pRecvPDU[CurrentIndex]->pBuffer = ((char *)(pRecvPDU[CurrentIndex]) +sizeof(PDU));;
					pRecvPDU[CurrentIndex]->uLength = MaxPacket;
					wsaBuf[CurrentIndex].len = pRecvPDU[CurrentIndex]->uLength;
					wsaBuf[CurrentIndex].buf = pRecvPDU[CurrentIndex]->pBuffer; 

					if ( WSARecv(hSocket, &wsaBuf[CurrentIndex], 1, NULL, &RecvFlags,
						&wsaRecvOverlapped[CurrentIndex], NULL) != 0)

					{
						iError = WSAGetLastError();
						if (iError != WSA_IO_PENDING)
						{
							//	printf("WSARecv failed: %d", iError);
							break;
						}
					}
					else
						bReceived = TRUE;

					CurrentIndex = (CurrentIndex +1)%MAX_RECV_PDUS;

					// Test close event while receiving
					if (WSAWaitForMultipleEvents(1, &pVC->hReceiveCloseEvent,FALSE,0,FALSE) == WAIT_OBJECT_0)
					{
						bReceived = FALSE;
						iError = ERROR_OPERATION_ABORTED;
						// printf ("Read operation Aborted\n");
						break;

					}
				}
				if (!bReceived)
				{
					if (iError == 0)
						iError = WSAGetLastError();
					if ( iError != WSA_IO_INCOMPLETE && iError != WSA_IO_PENDING)
					{
						break;
					}
				}

			}
			else if (dwEvent ==  WAIT_OBJECT_0 + MAX_RECV_PDUS)
			{
				// close event.
				// printf ("Read operation Aborted\n"); 
				iError = ERROR_OPERATION_ABORTED;
				break;
			}

			else
			{
				iError = ERROR_INVALID_HANDLE_STATE;
				printf("WSAGetOverlappedResult failed: %d\n", iError);
				break;
			}
		}
	}

	// Free the allocated resources                                                     ---
	for (i = 0; i <MAX_RECV_PDUS; i++)
	{
		if (hRecvEventList[i])
		{
			WSACloseEvent(hRecvEventList[i]);
			hRecvEventList[i] = NULL;
		}
		if (pRecvPDU[i])
		{
			free(pRecvPDU[i]);
			pRecvPDU[i] = NULL;
		}
	}
	return iError;
}


BOOL NDISCreateATMPVC( PATM_PVC pVC)
{
	ATM_PVC_PARAMS 	pvcParams;
	PFLOWSPEC       pSendQos, pRecvQos;
	DWORD           outBytes;
	PATM_SETTINGS   pSettings;
	DWORD dwError;

	if ((pVC->hSocket=NDISOpenATMSocket(pVC->pAdapter)) == INVALID_HANDLE_VALUE)
		return FALSE;

	pSettings = &pVC->ATMSettings;
	memset( &pvcParams, 0, sizeof(ATM_PVC_PARAMS)) ;

	pvcParams.PvcConnectionId.VPI = pSettings->VcSettings.Pvc.Vpi;
	pvcParams.PvcConnectionId.VCI = pSettings->VcSettings.Pvc.Vci;
	pvcParams.PvcConnectionId.DeviceNumber =pVC->pAdapter->DeviceNumber;
	pvcParams.PvcQos.ProviderSpecific.buf = NULL;
	pvcParams.PvcQos.ProviderSpecific.len = 0;

	pRecvQos = &pvcParams.PvcQos.ReceivingFlowspec;
	pSendQos = &pvcParams.PvcQos.SendingFlowspec;

	// Initialize QoS Settings with default values : UBR without specified QoS

	pRecvQos->TokenRate           = QOS_NOT_SPECIFIED;
	pRecvQos->TokenBucketSize     = QOS_NOT_SPECIFIED;
	pRecvQos->PeakBandwidth       = QOS_NOT_SPECIFIED;
	pRecvQos->Latency             = QOS_NOT_SPECIFIED;
	pRecvQos->DelayVariation      = QOS_NOT_SPECIFIED;
	pRecvQos->ServiceType         = SERVICETYPE_BESTEFFORT;
	pRecvQos->MaxSduSize          = QOS_NOT_SPECIFIED;
	pRecvQos->MinimumPolicedSize  = QOS_NOT_SPECIFIED;

	pSendQos->TokenRate           = QOS_NOT_SPECIFIED;
	pSendQos->TokenBucketSize     = QOS_NOT_SPECIFIED;
	pSendQos->PeakBandwidth       = QOS_NOT_SPECIFIED;
	pSendQos->Latency             = QOS_NOT_SPECIFIED;
	pSendQos->DelayVariation      = QOS_NOT_SPECIFIED;
	pSendQos->ServiceType         = SERVICETYPE_BESTEFFORT;
	pSendQos->MaxSduSize          = pSettings->PacketSize;
	pSendQos->MinimumPolicedSize  = QOS_NOT_SPECIFIED;

	if ( pSettings->QualityOfServices == QOS_UBR && 
		pSettings->QosSettings.Ubr.PeakCellRate != 0)
	{
		pSendQos->PeakBandwidth = pSettings->QosSettings.Ubr.PeakCellRate*48;
	}
	else if ( pSettings->QualityOfServices == QOS_CBR)
	{
		pSendQos->ServiceType     = SERVICETYPE_GUARANTEED;
		pRecvQos->ServiceType     = SERVICETYPE_GUARANTEED;
		pSendQos->PeakBandwidth =  pSettings->QosSettings.Cbr.PeakCellRate*48;
	}
	else if ( pSettings->QualityOfServices  == QOS_VBR)
	{
		pSendQos->ServiceType     = SERVICETYPE_CONTROLLEDLOAD;
		pRecvQos->ServiceType     = SERVICETYPE_CONTROLLEDLOAD;
		pSendQos->PeakBandwidth   = pSettings->QosSettings.Vbr.PeakCellRate*48;
		pSendQos->TokenRate       = pSettings->QosSettings.Vbr.SubstainableCellRate*48;
		pSendQos->TokenBucketSize = pSettings->QosSettings.Vbr.MaxBurstSize*48;
	}

	if (( WSAIoctl((SOCKET)pVC->hSocket, SIO_ASSOCIATE_PVC, (LPVOID) &pvcParams,
		sizeof(pvcParams), NULL, 0, &outBytes, NULL, NULL))== SOCKET_ERROR)
	{
		dwError = WSAGetLastError();
		printf("WSAIoctl failed: %d\n", dwError);
		NDISCloseATMSocket(pVC->hSocket);
		return FALSE;
	}

	//	ctlParam = 65536;
	//	iResult = setsockopt( (SOCKET)pVC->hSocket , SOL_SOCKET, SO_RCVBUF, (char *)&ctlParam, sizeof(DWORD) );

	pVC->hReceiveCloseEvent = WSACreateEvent();
	if (pVC->hReceiveCloseEvent == NULL)
	{
		NDISDeleteATMPVC(pVC);
		return FALSE;
	}
	// Create the transmit and receive mutexes the first time the VC is opened
	if (pVC->hReceiveMutex == NULL) 
		MUTEX_CREATE(pVC->hReceiveMutex); 

	if (pVC->hTransmitMutex == NULL) 
		MUTEX_CREATE(pVC->hTransmitMutex); 

	if ((pVC->hReceiveThreadHandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)NDISReceiveThread, (LPVOID)pVC, 0, NULL)) == 0)
	{
		NDISDeleteATMPVC(pVC);
		return FALSE;
	}

	return TRUE;

}

BOOL NDISDeleteATMPVC( PATM_PVC pVC)
{
	HANDLE handle;
	int bResult= FALSE;

	MUTEX_LOCK(pVC->hTransmitMutex);		// Do not write to vc when we are closing it

	if (pVC->hSocket != INVALID_HANDLE_VALUE)
	{
		handle = (HANDLE)pVC->hSocket;

		NDISCloseATMSocket(handle);
		bResult = TRUE;
	}
	MUTEX_UNLOCK(pVC->hTransmitMutex);		

	// stop the receiving thread
	if (pVC->hReceiveThreadHandle)
	{
		// Signal the event to close the receive thread.
		SetEvent(pVC->hReceiveCloseEvent);

		// Wait for it to exit. 
		WaitForSingleObject(pVC->hReceiveThreadHandle, INFINITE);
	}
	return bResult;
}


int NDISWriteATMPVC( PATM_PVC pVC, char *pBuffer, int bufSize)
{
	DWORD dwResult;
	OVERLAPPED oXmitOverlapped;
	HANDLE	   xmitEventList[1];

	int iReturn = -2;
	WSABUF xmitPacket;

	if(	pVC->bOpened && pVC->hSocket != INVALID_HANDLE_VALUE)
	{
		SOCKET hSocket = (SOCKET)pVC->hSocket;
		DWORD xmitFlags = ZERO_FLAGS;
		xmitPacket.len = bufSize;
		xmitPacket.buf = pBuffer;
		memset(&oXmitOverlapped, 0,sizeof(OVERLAPPED));
		oXmitOverlapped.hEvent= xmitEventList[0]=CreateEvent(0, FALSE, FALSE, NULL);

		if (WSASend( hSocket, &xmitPacket,1, &iReturn, xmitFlags,
			&oXmitOverlapped, NULL) != 0)
		{

			int iError = WSAGetLastError();
			if (iError != WSA_IO_PENDING)
			{
				CloseHandle(oXmitOverlapped.hEvent);
				iReturn = -6;
			}
			else
			{
				dwResult= WSAWaitForMultipleEvents( 1, xmitEventList, FALSE,WSA_INFINITE, FALSE);
				if (dwResult == WAIT_OBJECT_0) // close event;
				{
					iReturn =  0;

				}
				else if (dwResult <=  (WAIT_OBJECT_0 + 1 ))
				{
					xmitFlags= ZERO_FLAGS;
					dwResult=WSAGetOverlappedResult(hSocket, &oXmitOverlapped, &iReturn, TRUE, &xmitFlags);
					if (dwResult == FALSE)
						iReturn = -5;
				}
				else
				{
					iReturn = -7;
				}
				CloseHandle(oXmitOverlapped.hEvent);

			}
		}
	}
	return iReturn;
}