
/*
 * Example of standardized API with Prosum ATM drivers
 * 
 */

#include <winsock2.h>
#include <windows.h>
//#include <ansi_c.h> // for LabWindows environment
#include <stdio.h>
#include "proatmapi.h"
#include "wdmatmif.h"
#include "ndisatmif.h"


static TYPE_ENTRY proatmEntry[2]=
{{NDISGetATMAdapters, NDISGetATMInformation, NDISCreateATMPVC, NDISDeleteATMPVC, NDISWriteATMPVC},
 {WDMGetATMAdapters,  WDMGetATMInformation,  WDMCreateATMPVC,  WDMDeleteATMPVC,  WDMWriteATMPVC}};
 
ATM_ADAPTER proatmAdapterList[MAX_ATM_ADAPTERS*2];
int proatmAdapterCount = 0;

ATM_PVC vcList[MAX_VCS];
HANDLE  hOpenCloseMutex;



/*
    This function allocates a structure object for use
    in sending device requests

Arguments:
    Length : length of request parameters

Return Value:
    SUCCESS - returns PROATM object
    FAILURE - NULL

*/


LPPROTOCOL_DRIVER_REQUEST PROATMAllocateRequestData( int Length )
{
	LPPROTOCOL_DRIVER_REQUEST pPROATMRequest;

	pPROATMRequest = (LPPROTOCOL_DRIVER_REQUEST) calloc(  1,sizeof(PROTOCOL_DRIVER_REQUEST));

	if ( pPROATMRequest == NULL ) 
	{
		return NULL;
	}

	pPROATMRequest->OverLapped.hEvent = CreateEvent( NULL, FALSE, FALSE,  NULL );

	if ( pPROATMRequest->OverLapped.hEvent == NULL )
	{
		free(pPROATMRequest);
		return NULL;
	}

	pPROATMRequest->Buffer	  		= NULL;
	pPROATMRequest->Length	  		= 0;
	pPROATMRequest->ulBytesReceived	= 0;
	pPROATMRequest->bIoComplete		= FALSE;

	if (Length)
		pPROATMRequest->Buffer = (PVOID) calloc( 1,Length );
	if (pPROATMRequest->Buffer)
		pPROATMRequest->Length = Length;

	return pPROATMRequest;
}

/*
    Free the data allocated for device io control requests

Arguments:
    Length : length of request parameters

Return Value:
    SUCCESS - returns PROATM object
    FAILURE - NULL

*/


VOID  PROATMFreeRequestData( LPPROTOCOL_DRIVER_REQUEST pPROATMRequest )
{
	if (pPROATMRequest)
	{
		if ( pPROATMRequest->Buffer )
			free( pPROATMRequest->Buffer );

		CloseHandle(pPROATMRequest->OverLapped.hEvent);

		free(pPROATMRequest);
    }
}




/*
    This function sends an ioctl request to the adapter

Arguments:
    handle			- handle Object return by WDM
    pPROATMRequest   - PROATM object returned by PROATMAllocateRequestData
    bSync            - TRUE if service should wait for PROATM driver completion

Return Value:
    SUCCESS - TRUE if succeeded and SYNC==TRUE
    FAILURE -

*/
int PROATMDeviceIoControl( HANDLE handle, LPPROTOCOL_DRIVER_REQUEST pPROATMRequest,  ULONG ulIoctl,  int bSync )
{
	int Result;
	DWORD dwError;

	if( pPROATMRequest == NULL )
		return FALSE;

	pPROATMRequest->OverLapped.Offset		= 0;
	pPROATMRequest->OverLapped.OffsetHigh	= 0;
	pPROATMRequest->ulBytesReceived		    = 0;

	if ( !ResetEvent( pPROATMRequest->OverLapped.hEvent ) ) 
	{
		pPROATMRequest->bIoComplete = FALSE;
		return FALSE;
	}

	Result = DeviceIoControl(	handle,
								ulIoctl, 
								pPROATMRequest->Buffer, 
								pPROATMRequest->Length, 
								pPROATMRequest->Buffer, 
								pPROATMRequest->Length, 
								&(pPROATMRequest->ulBytesReceived), 
								&(pPROATMRequest->OverLapped) );

	if ( !Result && bSync )
	{
		//  They want to wait
		if ( (dwError = GetLastError()) == ERROR_IO_PENDING )
		{
			Result = GetOverlappedResult(	handle,
											&(pPROATMRequest->OverLapped),
											&(pPROATMRequest->ulBytesReceived),
											TRUE );
		}
	} 

	pPROATMRequest->bIoComplete = Result;
	return Result;
}


int proatmInitAdapters()
{
	int i;
	int AdapterCount= 0;
	if( proatmAdapterCount ==0)
	{
		memset(vcList,0, sizeof(vcList));
		for ( i=0; i< 2; i++)
		{
			proatmEntry[i].GetATMAdapters(proatmAdapterList, &AdapterCount);

		}
		if (AdapterCount > 0)
		{
			int i;

			if (AdapterCount > MAX_ATM_ADAPTERS*2)
				AdapterCount = MAX_ATM_ADAPTERS*2;

			for(i=0; i < AdapterCount; i++)
			{
				int type = proatmAdapterList[i].AdapterType; 
				// Get some information on the first adapter 
				if (!proatmEntry[type].GetATMInformation(&proatmAdapterList[i]))
					break;
			}
			proatmAdapterCount = i;
		}
	}

    MUTEX_CREATE(hOpenCloseMutex)
 
	return proatmAdapterCount;

}

void proatmExitAdapters()
{
	int i=0;

	for (i=0; i<MAX_VCS; i++)
	{
		proatmCloseVC(i);

		MUTEX_DELETE(vcList[i].hReceiveMutex);
		MUTEX_DELETE(vcList[i].hTransmitMutex);
	}
	
	MUTEX_DELETE(hOpenCloseMutex);

}


int proatmOpenVC(int adapterNumber, int vpi, int vci)
{
	PATM_PVC     pVC;
	BOOL bFoundVC = FALSE;
	int type = proatmAdapterList[adapterNumber].AdapterType; 

	int VCindx = -3; // No available VCs
	int i;

	if (adapterNumber >= proatmAdapterCount) 
		return -1; // Invalid adapter

	if (vpi < MIN_VPI || vpi> MAX_VPI || vci< MIN_VCI || vci >MAX_VCI )
		return -2; // Invalid vpi, vci

	MUTEX_LOCK(hOpenCloseMutex);

	// Check that this VC is not already open. 
	// Otherwise use the first available record of VCList
	do
	{
		// Scan the list of VCs
		for (i=0; i<MAX_VCS; i++)
		{
			if ( vcList[i].bOpened)
			{
				// We found an open VC. Is it the VC we gor in arguments?
				if ( vcList[i].pAdapter &&  
					 vcList[i].pAdapter->AdapterNumber == adapterNumber && 
					 vcList[i].ATMSettings.VcSettings.Pvc.Vpi == vpi && 
					 vcList[i].ATMSettings.VcSettings.Pvc.Vci == vci)
				{
					bFoundVC = TRUE;
					VCindx = i; // VC already opened, return the index in the vc table
					break;
				}
			}
			else if (VCindx < 0)
				VCindx = i;    // get the first free index 
		}

		
		if (bFoundVC || VCindx <0  )
			break;    // VC already opened no more available vcs

		pVC= &vcList[VCindx];

		INIT_LIST_HEAD(&pVC->ReceiveQueue);

		pVC->pAdapter = &proatmAdapterList[adapterNumber];
		pVC->ATMSettings.VcSettings.Pvc.Vpi = vpi;
		pVC->ATMSettings.VcSettings.Pvc.Vci = vci;

		pVC->ATMSettings.VcType = CONNECTION_PVC;
		pVC->ATMSettings.AalType = LAYER_AAL5;
		pVC->ATMSettings.SetQosSettings= TRUE;
		pVC->ATMSettings.QualityOfServices = QOS_UBR;
		pVC->ATMSettings.QosSettings.Ubr.PeakCellRate = 353208; // Max Rate
		pVC->ATMSettings.PacketSize = MAX_PDU_SIZE;
		pVC->ATMSettings.ReceiveFlow= TRUE;
		pVC->ATMSettings.XmitFlow   = TRUE;

		if (!proatmEntry[type].CreateATMPVC(pVC) )
			VCindx = -4; // createVC failed
		else
			pVC->bOpened = TRUE;

	}while(FALSE);

	MUTEX_UNLOCK(hOpenCloseMutex);
	
	return VCindx;
}



int proatmCloseVC(int  VCindx)
{
	PATM_PVC     pVC;
	PLIST_HEAD pEntry;
	int type;

	if (VCindx < 0 || VCindx >=MAX_VCS)
		return -1;

	pVC= &vcList[VCindx];

	MUTEX_LOCK(hOpenCloseMutex);

	if(pVC->bOpened)
	{
		type = pVC->pAdapter->AdapterType;
		proatmEntry[type].DeleteATMPVC(pVC);
		
		MUTEX_LOCK(pVC->hReceiveMutex);								// Protect the receive queue while it is emptying
		while ((pEntry = LIST_FIRST(&pVC->ReceiveQueue)) != NULL)
		{	
			PPDU pPDU= LIST_ENTRY_T(pEntry,PDU,Link);
			LIST_DEL(pEntry);
			free(pPDU);
		}
		MUTEX_UNLOCK(pVC->hReceiveMutex);

		pVC->bOpened = FALSE;

	}
	MUTEX_UNLOCK(hOpenCloseMutex);


	return 0;
}

/*
 Dequeue the received buffers from the VC receive queue
 Free the received buffers
 Return the length of received buffer
 
 */

int proatmReadPDU(int VCindx, char *pBuffer, int pduSize)
{
	PATM_PVC pVC;
	PLIST_HEAD pEntry;
	PPDU pPDU;

	if (VCindx < 0 || VCindx >=MAX_VCS)
		return -1;

	pVC= &vcList[VCindx];
	if(	!pVC->bOpened || pVC->hSocket == INVALID_HANDLE_VALUE)
	{
		return -2;
	}

	MUTEX_LOCK(pVC->hReceiveMutex);
	pEntry = LIST_FIRST(&pVC->ReceiveQueue);
	if (pEntry)
	{
		int bufSize;
		LIST_DEL(pEntry);
		MUTEX_UNLOCK(pVC->hReceiveMutex);
		pPDU = LIST_ENTRY_T(pEntry, PDU, Link);
		bufSize = pduSize < (int)(pPDU->uLength)? pduSize: (int)(pPDU->uLength);
		memcpy(pBuffer, pPDU->pBuffer, bufSize);
		free(pPDU); // The received buffers must be fred
		return bufSize;
	}
	else
		MUTEX_UNLOCK(pVC->hReceiveMutex);

	return 0;	
}


int proatmWritePDU(int  VCindx, char *pBuffer, int pduSize)
{
	PATM_PVC pVC;
	int iReturn;
	int type;

	if (VCindx < 0 || VCindx >=MAX_VCS)
		return -1;

	pVC= &vcList[VCindx];
	type = pVC->pAdapter->AdapterType;
	
	MUTEX_LOCK(pVC->hTransmitMutex);		// Protect the transmit operation from intempestive closing
	iReturn =proatmEntry[type].WriteATMPVC(pVC, pBuffer, pduSize);
	MUTEX_UNLOCK(pVC->hTransmitMutex);		

	return iReturn;

}

