/*++
    Copyright (c) PROSUM.  All rights reserved.

    THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
    KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
    PURPOSE.

Module Name:

    public.h

Abstract:

    This module contains the common declarations shared by driver
    and user applications.

Environment:

    user and kernel

--*/




#define FSCTL_ATM_BASE      FILE_DEVICE_NETWORK

#define _ATM_CTL_CODE(_Function, _Method, _Access)  \
            CTL_CODE(FSCTL_ATM_BASE, _Function, _Method, _Access)

#define IOCTL_ATM_CREATE_VC   \
            _ATM_CTL_CODE(0x202, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)

#define IOCTL_ATM_CLOSE_VC   \
            _ATM_CTL_CODE(0x203, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)

#define IOCTL_ATM_QUERY_OID_VALUE   \
            _ATM_CTL_CODE(0x204, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)

#define IOCTL_ATM_SET_OID_VALUE   \
            _ATM_CTL_CODE(0x205, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)

#define IOCTL_ATM_INDICATE_STATUS   \
            _ATM_CTL_CODE(0x206, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)

//
//  Structure to go with IOCTL_ATM_QUERY_OID_VALUE.
//  The Data part is of variable length, determined by
//  the input buffer length passed to DeviceIoControl.
//
typedef struct _ATM_QUERY_OID
{
    ULONG        Oid;
    unsigned char        Data[sizeof(ULONG)];

} ATM_QUERY_OID, *PATM_QUERY_OID;

//
//  Structure to go with IOCTL_ATM_SET_OID_VALUE.
//  The Data part is of variable length, determined
//  by the input buffer length passed to DeviceIoControl.
//
typedef struct _ATM_SET_OID
{
    ULONG        Oid;
    unsigned char        Data[sizeof(ULONG)];

} ATM_SET_OID, *PATM_SET_OID;


//
//  Structure to go with IOCTL_ATM_INDICATE_STATUS.
//  ATM copies the Status indicated by the NIC and
//  also the data indicated in the StatusBuffer.
//
typedef struct _ATM_INDICATE_STATUS
{
    ULONG            IndicatedStatus;        // NDIS_STATUS
    ULONG            StatusBufferOffset;    // from start of this struct
    ULONG            StatusBufferLength;    // in bytes
} ATM_INDICATE_STATUS, *PATM_INDICATE_STATUS;
 



//
// Define an Interface Guid for PROATM device class.
// This GUID is used to register (IoRegisterDeviceInterface)
// an instance of an interface so that user application
// can control the PROATM device.
//
// {FB9E31D1-012B-46bd-9462-450E7796080E}
DEFINE_GUID(GUID_DEVINTERFACE_PROATM, 
0xfb9e31d1, 0x12b, 0x46bd, 0x94, 0x62, 0x45, 0xe, 0x77, 0x96, 0x8, 0xe);
        
//
// Define a WMI GUID to get PROATM device info.
//

DEFINE_GUID (PROATM_WMI_STD_DATA_GUID,
    0x20e35e40, 0x7179, 0x4f89, 0xa2, 0x8c, 0x12, 0xed, 0x5a, 0x3c, 0xaa, 0xa5);

// {20E35E40-7179-4f89-A28C-12ED5A3CAAA5}

//
// WMI GUID to set/get conservation idle time. The ConservationIdleTime value
// applies when the system power policy optimizes for conservation.
// Typically, the applicable policy depends upon the power source:
// when running with AC power, the system optimizes for performance, and when
// running off a battery, the system optimizes for conservation.
//

DEFINE_GUID (GUID_POWER_CONSERVATION_IDLE_TIME,
    0x106a479, 0x477c, 0x4aef, 0x91, 0xc2, 0xdd, 0x96, 0x66, 0xd0, 0xf4, 0x9);

// {0106A479-477C-4aef-91C2-DD9666D0F409}


//
// WMI GUID to set/get performance idle time. The PerformanceIdleTime value
// applies when the system power policy optimizes for performance.
// Typically, the applicable policy depends upon the power source:
// when running with AC power, the system optimizes for performance, and when
// running off a battery, the system optimizes for conservation.
//

DEFINE_GUID (GUID_POWER_PERFORMANCE_IDLE_TIME,
    0x3e426488, 0x5b3a, 0x4353, 0xb0, 0xfc, 0x53, 0x7f, 0xe8, 0xf8, 0x2d, 0x7);
// {3E426488-5B3A-4353-B0FC-537FE8F82D07}

//
// GUID definition are required to be outside of header inclusion pragma to avoid
// error during precompiled headers.
//

